function [ BpostDAS, beamformed, CF ] = beamformCF( RF, fulAprSz, probeL )
%BEAMFORMCF Summary of this function goes here
%   Detailed explanation goes here

% dasBeamform = beamformDAQ(RF, fulAprSz, probeL);
% BpostDAS = abs(hilbert(dasBeamform));

hlfAprSz = floor(fulAprSz/2);
fulAprSz = hlfAprSz*2 + 1;

win = hanning(fulAprSz)';   % windowing used for apodization

channelSpacing = probeL/128;
sampleSpacing = 1/40e6*1540*1e3; % spacing between samples in photoacoustic imaging

offset = 10;             % distance between first sample and transducer element in milimeter
nSampleOffset = round(offset/ sampleSpacing);    % offset in samples: 1st sample is actually 1+nSampleOffset sample

[ns, nl] = size(RF);

for i = 1:nl    % for each line/channel
%     disp(['scanline #', num2str(i)]);
    
    for j=1:ns  % find the value for each sample
        
        % calc delays based on sample depth and receive aperture
        distance2probe =  (j + nSampleOffset) * sampleSpacing ; 
        rad = sqrt( (hlfAprSz*channelSpacing)^2 +  distance2probe^2 );
        scale = (rad - distance2probe);
        delays = + x.^2 / (hlfAprSz)^2 * scale/ sampleSpacing  ; 

        cntr = i;       % center of apreture
        apr = cntr + x; % full aperture indices
        
        chnls = zeros(1,fulAprSz);  % initialize channel values
        cnt = 0;            % count the number of channel that we use
                            % this parameter can be used for averaging (not used for now)

        % find the corresponding value from each channel
        for k = 1:fulAprSz  
            
            chlIndx = apr(k);
            if chlIndx<1, continue, end;
            if chlIndx>nl, continue, end;
            cnt = cnt + 1;

            chlSmpl = round(j+delays(k));

            if chlSmpl<1, continue, end;
            if chlSmpl>ns, continue, end;
            
            chnls(k) = preBeamformed(chlIndx, chlSmpl); 
            
        end;
        % apodization : ideally has to be a function of depth
        chnls = win .* chnls;
        
        % beamfroming
        out(i,j) = sum( chnls ) ;%/ cnt;
        
    end;
end

BpostDAS = BpostDAS/max(max(BpostDAS));

beamformed = zeros(ns,nl);

CF = coherenceFactor(BpostDAS);

for i = 1:1:ns
	beamformed(i,:) = CF(i)*BpostDAS(i,:);
end

end

