% Authors: Alexis Cheng
% Modified from Existing Code from Steven Su, Robert Kim, Nathanael Kuo
% Date: January 7, 2012
% Interventional Photoacoustic Registration
% 600.446: CIS II, Spring 2012
% The Johns Hopkins University

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Gets the centroids of the photoacoustic signal in the ultrasound images.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% function [us_coordB, us_coordK, BpostB, BpostK] = getUltrasoundPointAC(path)
close all;
clear all;

nCh = 128;              % number of channels
reRoute = true;         % true: transducer element (correct image), false: DAQ element
chanls = ones(1,nCh);   % what channels to read (DAQ element), for each channel set to 1
fulAprSz = 32;          % aperature size
dx = (38.4e-3)/128;
dz = 0.1;
dt = 1/(40e6);
speedSound = 1540;
stepSize = 1.384/2;
threshold = 0.2;

us_totalB = [];
us_tempB = [];
us_totalK = [];
us_tempK = [];
sum_regionK = 0;
xTotal = 0;
yTotal = 0;
zTotal = 0;
used = [];
usedCentroid = [];

xKTemp = [];
yKTemp = [];
idPix = [];
SurfaceProj = zeros(128,41);

path = 'C:\Users\Alexis\School\PA\06_Alexis\02_Data\2012.02.29-3DAMSExVivoData\FirstPoint\';
if (path(end) ~= '\') path = [path,'\']; end
% Sets correct directory
D = dir(path);
idx = find([D.isdir]);

for i = 3:1:idx(end)
    folder = D(i).name;
    filePath = [path,folder,'\'];
    thresholdK = 0;
    frameN = 3;
    
    while (thresholdK < threshold) && (frameN < 4)
        [hdr, RFslice] = readDAQ(filePath, chanls, frameN, reRoute); % reads DAQ data
        
        RF(:,:,i-2) = RFslice;
        
        
        
        y = [1 size(RFslice,1)] / 40e6 * 1540 * 1e3;
        x = [1 size(RFslice,2)] / 128 * 38.4;
        z = [1 size(RFslice,3)] * dz;
        
        Bpre = abs(hilbert(RF(:,:,i-2)));  % Computes prebeamform image
        
        p_zx = kspaceLineRecon(RF(:,:,i-2),dx,dt,speedSound);
        
        BpostK=abs(hilbert(p_zx));
        BpostKAll(:,:,i-2) = BpostK;
        %BpostK = 255*BpostK/max(max(BpostK));
        
        BpostK(1:50,:) = 0;
        BpostK(1950:2000,:) = 0;
        
        grayBpostK = mat2gray(BpostK);
        thresholdK = graythresh(grayBpostK)
        
        thresholdK = 0.25;
        frameN = frameN + 1;
    end
    
    if (thresholdK < threshold)
        thresholdK = 0.99;
    end
    
    bwBpostK = im2bw(grayBpostK,thresholdK);
    
    min_pixel_count = 150;
    max_pixel_count = 1000;
    
    minBW_USimageK = bwareaopen(bwBpostK,min_pixel_count,8);
    maxBW_USimageK = ~bwareaopen(bwBpostK,max_pixel_count,8);
    BW_USimageK = minBW_USimageK & maxBW_USimageK;

%     figure;
%      subplot(131);
%     imagesc(x,y,Bpre);
%         title({'Pre-Beamform'; 'PA Signal Image'});
%     xlabel('Lateral (mm)');
%     ylabel('Axial (mm)');
%     colormap(gray);
%     axis equal;
%     axis tight;
%     
%      subplot(132);
%     imagesc(x,y,BpostK);
%         title({'k-Wave Beamformed'; 'PA Signal Image'});
%     xlabel('Lateral (mm)');
%     ylabel('Axial (mm)');
%     colormap(gray);
%     axis equal;
%     axis tight;
% 
%      subplot(133);
%     imagesc( x,y,BW_USimageK);
%     title({'Segmented PA'; 'Signal Image'});
%     xlabel('Lateral (mm)');
%     ylabel('Axial (mm)');
%     colormap(gray);
%     axis equal;
%     axis tight;
    
    [B,LK] = bwboundaries(BW_USimageK);
    statsK = regionprops(LK, 'all');
    
    if ~isempty(statsK)
        for j = length(statsK): -1 : 1
            if ((statsK(j).Area < min_pixel_count) || (statsK(j).Area > max_pixel_count))
                statsK(j,:) = [];
            end
        end
        
        if (length(statsK) > 1)
            continue;
        end
        
        dimensions = size(RF);
        
        sizePixel = size(statsK.PixelList);
        for m = 1:1:sizePixel(1)-1
            if(statsK.PixelList(m,1) < statsK.PixelList(m+1,1))
                idPix = [idPix; statsK.PixelIdxList(m)];
                sum_regionK = sum_regionK + 1;
                xKTemp = [xKTemp; x(2) * statsK.PixelList(m,1) / dimensions(2)];
                yKTemp = [yKTemp; y(2) * statsK.PixelList(m,2) / dimensions(1)];
            end
        end
%         sum_regionK = sum_regionK + sum(BpostK(idPix));
        zKTemp = 0;
        [surfY, surfX] = ind2sub([2000 128],idPix);
        
        for k = 1:1:length(xKTemp)
            K(:,k) = Conv2DTo3DProbe([xKTemp(k) yKTemp(k) zKTemp], i-3, stepSize);
            SurfaceProj(surfX(k), i-2) = BpostK(surfY(k), surfX(k));
            
%             figure(1);
% %             plot3(K(1,k), K(2,k), K(3,k),'.');
%             hold on;
        end
        
%         xTotal = xTotal + sum(K(1,:)' .* double(BpostK(idPix)));
%         yTotal = yTotal + sum(K(2,:)' .* double(BpostK(idPix)));
%         zTotal = zTotal + sum(K(3,:)' .* double(BpostK(idPix)));
        xTotal = xTotal + sum(K(1,:));
        yTotal = yTotal + sum(K(2,:));
        zTotal = zTotal + sum(K(3,:));
        i-2
        used = [used; i-2];
        usedCentroid = [usedCentroid; statsK.Centroid];
        
        K = [];
        xKTemp = [];
        yKTemp = [];
        idPix = [];
    end
    
end

newimage = max(BpostKAll,[],1);
figure;
imagesc(squeeze(newimage));
colormap(gray);

xBar = xTotal / sum_regionK;
yBar = yTotal / sum_regionK;
zBar = zTotal / sum_regionK;

wCentroidK = [xBar yBar zBar];

us_coordK = wCentroidK'

figure;
imagesc(SurfaceProj);
colormap(gray);
hold on;
used