% Authors: Alexis Cheng
% Modified from Existing Code from Steven Su, Robert Kim, Nathanael Kuo
% Date: January 7, 2012
% Interventional Photoacoustic Registration
% 600.446: CIS II, Spring 2012
% The Johns Hopkins University

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Gets the centroids of the photoacoustic signal in the ultrasound images.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% function [us_coordB, us_coordK, BpostB, BpostK] = getUltrasoundPointAC(path)
% close all;
% clear all;

stepSize = 1.384/2;
min_pixel_count = [100 100 150 150 150 100 150];
max_pixel_count = 1000;
threshold = [0.25 0.2 0.25 0.2 0.2 0.25 0.25];
depth = 1;

sum_regionK = 0;
xTotal = 0;
yTotal = 0;
zTotal = 0;
used = [];
usedCentroid = [];

us_coordK = zeros(3,7);
us_coordMax = zeros(3,7);
us_coordCorr = zeros(3,7);

plotFig = [0 0 0 0];

xKTemp = [];
yKTemp = [];
idPix = [];
SurfaceProj = zeros(128,41,7);
SurfaceProjOld = zeros(128,41,7);
maxBpostKAll = zeros(128,41,7);
meanBpostKAll = zeros(128,41,7);
sumBpostKAll = zeros(128,41,7);

maskIndex = [128 1;41 1];
tempID = [];
slice = [];
frame = 3;
pathStrings = {'First' 'Second' 'Fifth' 'Sixth' 'Seventh' 'Ninth' 'Tenth'};

for imNum = 1:1:7
    path = ['C:\Users\Alexis\School\PA\06_Alexis\02_Data\2012.02.29-3DAMSExVivoData\'  char(pathStrings(imNum))  'Point\'];
    % Sets correct directory
    D = dir(path);
    idx = find([D.isdir]);
    startIndex = [1 20 8 1 1 15 10];
    endIndex = [idx(end)-2 28 22 idx(end)-2 idx(end)-2 25 23];
    
    [ RF(:,:,:,imNum), p_zx(:,:,:,imNum), BpostKAll(:,:,:,imNum), x(imNum), y(imNum), z(imNum) ] = beamformVolume( path, frame);
    
    meanBpostKAll(:,:,imNum) = meanAxial(BpostKAll(:,:,:,imNum));
    maxBpostKAll(:,:,imNum) = maxAxial(BpostKAll(:,:,:,imNum));
    sumBpostKAll(:,:,imNum) = sumAxial(BpostKAll(:,:,:,imNum));
    
    if (plotFig(1))
        plotUSImage(meanBpostKAll(:,:,imNum),'Mean',0);
        plotUSImage(maxBpostKAll(:,:,imNum),'Max',0);
        plotUSImage(sumBpostKAll(:,:,imNum),'Sum',0);
    end
    
    thresholdK = threshold(imNum);
    grayBpostKAll = mat2gray(BpostKAll);
   
    for i = startIndex(imNum):1:endIndex(imNum)
        BpostK = BpostKAll(:,:,i,imNum);
        grayBpostK = grayBpostKAll(:,:,i,imNum);
        
        bwBpostK = im2bw(grayBpostK,thresholdK);
        
        BW_USimageK = restrictPixelCount(bwBpostK,min_pixel_count(imNum),max_pixel_count);
        
        [B,LK] = bwboundaries(BW_USimageK);
        statsK = regionprops(LK, 'all');
        
        if ~isempty(statsK)
            for j = length(statsK): -1 : 1
                if ((statsK(j).Area < min_pixel_count(imNum)) || (statsK(j).Area > max_pixel_count))
                    statsK(j,:) = [];
                end
            end       
            if (length(statsK) > 1)
                continue;
            end
            
            dimensions = size(RF);
            
            sizePixel = size(statsK.PixelList);
            for m = 1:1:sizePixel(1)-1
                %             if(statsK.PixelList(m,1) < statsK.PixelList(m+1,1))
                %                 for n = 1:1:depth
                %                     if (m-n+1 > 0)
                n = 1;
                idPix = [idPix; statsK.PixelIdxList(m-n+1)];
                sum_regionK = sum_regionK + 1;
                xKTemp = [xKTemp; x(2) * statsK.PixelList(m-n+1,1) / dimensions(2)];
                yKTemp = [yKTemp; y(2) * statsK.PixelList(m-n+1,2) / dimensions(1)];
                %                     end
                %                 end
                %             end
            end
            zKTemp = 0;
            
            [surfY, surfX] = ind2sub([2000 128],idPix);
            for j = 1:1:length(surfX)
                slice = [slice; i];
            end
            tempID = [tempID; surfY surfX slice];
            SurfaceProj(surfX(1),i,imNum) = BpostK(surfY(1), surfX(1));
            
            for k = 2:1:length(surfX)
                if (surfX(k) == surfX(k-1))
                    SurfaceProj(surfX(k),i,imNum) = SurfaceProj(surfX(k),i,imNum) + BpostK(surfY(k), surfX(k));
                else
                    SurfaceProj(surfX(k),i,imNum) = BpostK(surfY(k),surfX(k));
                end
                if (i < maskIndex(2,1))
                    maskIndex(2,1) = i;
                end
                if (i > maskIndex(2,2))
                    maskIndex(2,2) = i;
                end
                if (surfX(k) < maskIndex(1,1))
                    maskIndex(1,1) = surfX(k);
                end
                if (surfX(k) > maskIndex(1,2))
                    maskIndex(1,2) = surfX(k);
                end
            end
            
            for k = 1:1:length(xKTemp)
                K(:,k) = Conv2DTo3DProbe([xKTemp(k) yKTemp(k) zKTemp], i-1, stepSize);
                
                SurfaceProjOld(surfX(k), i,imNum) = BpostK(surfY(k),surfX(k));
                if (plotFig(2))
                    figure(nnz(plotFig)*(imNum-1)+1);
                    plot3(K(1,k), K(2,k), K(3,k),'.');
                    hold on;
                end
            end
            
            xTotal = xTotal + sum(K(1,:));
            yTotal = yTotal + sum(K(2,:));
            zTotal = zTotal + sum(K(3,:));
            
            K = [];
            xKTemp = [];
            yKTemp = [];
            idPix = [];
            slice = [];
        end
        
    end
    
    xBar = xTotal / sum_regionK;
    yBar = yTotal / sum_regionK;
    zBar = zTotal / sum_regionK;
    
    wCentroidK = [xBar yBar zBar];
    
    us_coordK(:,imNum) = wCentroidK';
    
    newimage = max(BpostKAll,[],1);
    
    % max intensity element on slice, get furthestmost axial point
    [surfMaxY surfMaxI] = max(SurfaceProj(:,:,imNum));
    targetX = max(surfMaxI);
    
    [surfMaxY surfMaxI] = max(max(SurfaceProj(:,:,imNum)));
    targetSlice = surfMaxI;
    
    for i = 1:1:length(tempID)
        if (tempID(i,3) == targetSlice)
            if (tempID(i,2) == targetX)
                targetY = tempID(i,1);
                index = i;
            end
        end
    end
    
    xKTemp = x(2)*tempID(i,2)/ dimensions(2);
    yKTemp = y(2)*tempID(i,1)/ dimensions(1);
    zKTemp = 0;
    
    us_coordMax(:,imNum) = Conv2DTo3DProbe([xKTemp yKTemp zKTemp], targetSlice - 1, stepSize);
    
    
    mask = SurfaceProj(maskIndex(1,1):maskIndex(1,2),maskIndex(2,1):maskIndex(2,2),imNum);
    
    correlationIndex = correlation(mask,squeeze(surfaceMaxProj));
    
    targetX = correlationIndex(1);
    targetSlice = correlationIndex(2);
    for i = 1:1:length(tempID)
        if (tempID(i,3) == targetSlice)
            if (tempID(i,2) == targetX)
                targetY = tempID(i,1);
                index = i;
            end
        end
    end
    
    xKTemp = x(2)*tempID(i,2)/ dimensions(2);
    yKTemp = y(2)*tempID(i,1)/ dimensions(1);
    zKTemp = 0;
    
    us_coordCorr(:,imNum) = Conv2DTo3DProbe([xKTemp yKTemp zKTemp], targetSlice - 1, stepSize);
    
    if (plotFig(2))
        plot3(us_coordK(1,imNum), us_coordK(2,imNum),us_coordK(3,imNum) ,'*r');
        plot3(us_coordMax(1,imNum), us_coordMax(2,imNum), us_coordMax(3,imNum), '*g');
        plot3(us_coordCorr(1,imNum), us_coordCorr(2,imNum), us_coordCorr(3,imNum), '*c');
    end
    if (plotFig(3))
        plotUSImage(SurfaceProj(:,:,imNum),'Sum of Region pixels',1);
    end
    if(plotFig(4))
        plotUSImage(SurfaceProjOld(:,:,imNum),'Surface Pixel - Depth 1',1);
    end
    
    xKTemp = [];
    yKTemp = [];
    xTotal = 0;
    yTotal = 0;
    zTotal = 0;
end