% Authors: Alexis Cheng
% Modified from Existing Code from Steven Su, Robert Kim, Nathanael Kuo
% Date: January 7, 2012
% Interventional Photoacoustic Registration
% 600.446: CIS II, Spring 2012
% The Johns Hopkins University

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Gets the centroids of the photoacoustic signal in the ultrasound images.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% function [us_coordB, us_coordK, BpostB, BpostK] = getUltrasoundPointAC(path)
close all;
clear all;

nCh = 128;              % number of channels
reRoute = true;         % true: transducer element (correct image), false: DAQ element
chanls = ones(1,nCh);   % what channels to read (DAQ element), for each channel set to 1
fulAprSz = 32;          % aperature size
dx = (38.4e-3)/128;
dz = 0.1;
dt = 1/(40e6);
speedSound = 1540;
frameN = 1;

us_totalB = [];
us_tempB = [];
us_totalK = [];
us_tempK = [];
counter=0;

path = 'C:\Users\Alexis\School\PA\06_Alexis\02_Data\2012.02.16-3DUSData\SeventhPoint\';
if (path(end) ~= '\') path = [path,'\']; end
% Sets correct directory
D = dir(path);
idx = find([D.isdir]);

for i = 3:idx(end)
    folder = D(i).name;
    filePath = [path,folder,'\'];
    
    [hdr, RFslice] = readDAQ(filePath, chanls, frameN, reRoute); % reads DAQ data
    
    RF(:,:,i-2) = RFslice;
    
    y = [1 size(RFslice,1)] / 40e6 * 1540 * 1e3;
    x = [1 size(RFslice,2)] / 128 * 38.4;
    z = [1 size(RFslice,3)] * dz;
end

%Bpre = abs(hilbert(RF));  % Computes prebeamform image

%out = beamformDAQ(RF, fulAprSz); % Computes beamformed image
%BpostB = abs( hilbert(out) );  % Processes beamformed image

%p_zx = kspaceLineRecon(RF,dx,dt,speedSound);
p_zxy = kspacePlaneRecon(RF,dx,dz,dt,speedSound);

for i = 3:idx(end)
    
    p_zxyNew(i-2,:,:) = p_zxy(:,:,i-2);
    
end

%BpostK=abs(hilbert(p_zx));

figure;
title('Bpre, Bpost, Kwave Images');
xlabel('x (mm)');
ylabel('y (mm)');

subplot(131);
imagesc(x,y,Bpre);
colormap(gray);
axis equal;

subplot(132);
imagesc( x,y,BpostB);
colormap(gray);
axis equal;

subplot(133);
imagesc(x,y,BpostK);
colormap(gray);
axis equal;

grayBpostB = mat2gray(BpostB);
thresholdB = graythresh(grayBpostB);
bwBpostB = im2bw(grayBpostB,thresholdB);

grayBpostK = mat2gray(BpostK);
thresholdK = graythresh(grayBpostK);
bwBpostK = im2bw(grayBpostK,thresholdK);

figure;
title('bwBpost, bwKwave');
xlabel('x (mm)');
ylabel('y (mm)');

subplot(121);
imagesc(x,y,bwBpostB);
colormap(gray);
axis equal;

subplot(122);
imagesc(x,y,bwBpostK);
colormap(gray);
axis equal;

min_pixel_count = 100;
max_pixel_count = 2000;

minBW_USimageB = bwareaopen(bwBpostB,min_pixel_count,8);
maxBW_USimageB = ~bwareaopen(bwBpostB,max_pixel_count,8);
BW_USimageB = minBW_USimageB & maxBW_USimageB;

minBW_USimageK = bwareaopen(bwBpostK,min_pixel_count,8);
maxBW_USimageK = ~bwareaopen(bwBpostK,max_pixel_count,8);
BW_USimageK = minBW_USimageK & maxBW_USimageK;

figure;
title('Segmented Bpost, Segmented Kwave Images - Pixel count 50-1000');
xlabel('x (mm)');
ylabel('y (mm)');

subplot(121);
imagesc( x,y,BW_USimageB);
colormap(gray);
axis equal;

subplot(122);
imagesc( x,y,BW_USimageK);
colormap(gray);
axis equal;

% TRY USING EXTENT!!

[B,LB] = bwboundaries(BW_USimageB);
statsB = regionprops(LB, 'all');

for i = length(statsB): -1 : 1
    if ((statsB(i).Area < min_pixel_count) || (statsB(i).Area > max_pixel_count))
        statsB(i,:) = [];
    end
end

for i = 1 : length(statsB)
    idx = statsB(i).PixelIdxList;
    sum_regionB = sum(BpostB(idx));
    xB = statsB(1).PixelList(:, 1);
    yB = statsB(1).PixelList(:, 2);
    xbar = sum(xB .* double(BpostB(idx))) / sum_regionB;
    ybar = sum(yB .* double(BpostB(idx))) / sum_regionB;
    %centroidB = statsB(i).Centroid;
    wCentroidB = [xbar ybar];
    us_tempB = [us_tempB; wCentroidB];
end

[B,LK] = bwboundaries(BW_USimageK);
statsK = regionprops(LK, 'all');

for i = length(statsK): -1 : 1
    if ((statsK(i).Area < min_pixel_count) || (statsK(i).Area > max_pixel_count))
        statsK(i,:) = [];
    end
end

for i = 1 : length(statsK)
    idx = statsK(i).PixelIdxList;
    sum_regionK = sum(BpostK(idx));
    xK = statsK(1).PixelList(:, 1);
    yK = statsK(1).PixelList(:, 2);
    xbar = sum(xK .* double(BpostK(idx))) / sum_regionK;
    ybar = sum(yK .* double(BpostK(idx))) / sum_regionK;
    %centroidK = statsK(i).Centroid;
    wCentroidK = [xbar ybar];
    us_tempK = [us_tempK; wCentroidK];
end

% if frameN == 1
us_totalB = us_tempB;
us_totalK = us_tempK;
% else
%     us_totalB = us_totalB + us_tempB;
%     us_totalK = us_totalK + us_tempK;
% end

counter = counter + 1;
% end

% Computes the mean centroid for the image based on all the frames.
us_avgB = us_totalB/counter;
us_avgK = us_totalK/counter;

us_avgB = [us_avgB 0]';
us_avgK = [us_avgK 0]';

dimensions = size(RF);

fractionB = [us_avgB(1)/dimensions(2) us_avgB(2)/dimensions(1)];
fractionK = [us_avgK(1)/dimensions(2) us_avgK(2)/dimensions(1)];

us_coordB = [fractionB(1)*x(2) fractionB(2)*y(2) 0]'
us_coordK = [fractionK(1)*x(2) fractionK(2)*y(2) 0]'