function [out spacing_z] = linear2Sector(img, radios, ang, x_spacing, set_y)
%  Original             Scan Converted Image
%   ______               _________ 
%  |      |             |         |
%  |      |              |       |
%  |      |               |     |
%  |      |                |___|
%  |______|                  
%
% usage    newImg = steeredImage(oldImg, probeRadios, probeAngle)
%
% Note: no interpolation or averaging is applied to the scanconverted image
%
% Original Author: Reza Zahiri Azar (c) 
% Ultrasonix Medical Corporation Oct 2006
% email: reza.zahiri@ultrasonix.com
% Author: Nishikant Deshmukh (c) MUSiiC Lab www.musiic.org
% email: nishikant@jhu.edu

probeAng = ang * pi/180;
probeRad = round(radios);

% x axial direction y lateral direction
[x y] = size(img);
% the image expands in axial direction
x2 = round(probeRad + x);    

x2_mm = x2 * x_spacing;

if radios > 0
    cutoff = round(probeRad * cos(probeAng/2));
    cutoff_mm = cutoff * x_spacing;
    %cutoff = 0;
else 
    cutoff = 0;
end

% center of lateral
center = round(x2*sin(probeAng/2));

center_mm = round (x2_mm * sin (probeAng/2));

% depending on the angle and #of samples the image expands or shrinks in lateral direction
y2 = 2*center;    

y2_mm = 2 * center_mm;

new_y = set_y;

new_x = round((x2_mm-cutoff_mm)/y2_mm * new_y);

% new_cutoff = round (cutoff * new_x / x2);
% allocate memory for the new image unused pixels will be black
out = zeros(new_x,new_y);

% angle step
angStep = probeAng/y;
for i = 1:x
    for j = 1:y
        % transfer to polar cordinate i axial and j lateral direction
        r = probeRad + i;
        theta = -probeAng/2 + j*angStep;
        % transfer to cartesian coordinates
        i2 = round(r*cos(theta)-cutoff); 
%         i2 = i2 - cutoff;
%         if (i2 < 0) 
%             i2 = 0;
%         end
        i3 = ceil(i2 * new_x/(x2-cutoff));
        % for lateral first move to the center of image
        j2 = center + round(r*sin(theta));
        % transfer to new image (no averaging or interpolation)
        j3 = round (j2 * new_y/y2);
        out(i3 ,j3) = img(i,j);
    end;
end;
spacing_z = y2_mm / size(out,2);




